/****************************************************

		concen.c
   
   These functions generate concentricity vectors for
   the specified fibers
   the values 
   to be loaded and the data placed in the button
   eccentricity structure.

   Functions in this file include:

   *  FILE *eccenOpen()
   *  eccenLoad()
   *  eccenCalcVec()

   ( * indicates that function is declared as an extern )

****************************************************/

#include <stdio.h>
 
#include "miscMath.h"
#include "motor.h"
#include "kaboom.h"
#include "plate.h"
#include "translate.h"
#include "flex.h"
#include "buttons.h"
#include "concen.h"


int EccenFlag = 0;	/*  Flag to apply eccentricity correction */



void eccenCalcVec( struct fiber_dbase *pFib,
                   struct aPoint *target,
		   struct aPoint *vector )

{
   double theta;
   double phi;
   double psi;
   double d1;
   double d2;
   double d3;
   double tmpAngle;
   double buttonAngle;
   double deflection;

   /* some of this was lifted from kaboom.c */

   /* calculate the angle of the radial line wrt 0 */

   theta = atan2( pFib->CurGeometry->pivot.y, pFib->CurGeometry->pivot.x );

   /* calculate the distance from the button to the origin */

   d2    = sqrt(  target->x * target->x + target->y * target->y );

   /* calculate the angle at the origin between the button
      and the radial line, retaining sign convention even
      when the angle crosses the +/- PI boundry. */

   if( ( target->x == 0 ) && ( target->y == 0 ) ){

      phi = 0;
   }

   else{

      tmpAngle = atan2( target->y, target->x );

      if( theta >= 0 ){
	 if( tmpAngle <= 0 )
	    tmpAngle += TWOPI;
      }
      else
	 if( tmpAngle  >= 0 )
	    tmpAngle -= TWOPI;

      phi = tmpAngle - theta;

      if( phi < -M_PI )
	 phi += TWOPI;

      if( phi > M_PI )
	 phi -= TWOPI;
   }

   
   /* distance along radial line from origin to button */

   d3    = d2 * cos( phi );

   /* distance along radial line from pivot to button */

   d1    = PIVOT - d3 ;

   /* deflection of button from radial line */

   deflection  = d2 * sin( phi );

   /* angle between radial line and button */

   psi   = atan2( deflection, d1 );

   /* rotation of button due to deflection */

   buttonAngle = psi * DEFLECTION_RATIO;

   /* calculate the absolute angle of the eccentricity
      vector wrt origin */

   tmpAngle = theta + buttonAngle + pFib->Eccentricity.angle;

   /* calculate x,y components */

   vector->x = pFib->Eccentricity.mag * cos( tmpAngle );
   vector->y = pFib->Eccentricity.mag * sin( tmpAngle );

   /* invert vector to get the correction. */

   vector->x = -( vector->x );
   vector->y = -( vector->y );
}





